package geniusweb.boa.biddingstrategy;

import geniusweb.actions.Action;
import geniusweb.actions.PartyId;
import geniusweb.boa.BoaParty;
import geniusweb.boa.NegoState;
import geniusweb.boa.OpponentModel;
import geniusweb.boa.acceptancestrategy.AcceptanceStrategy;
import geniusweb.party.Party;
import geniusweb.progress.Progress;
import tudelft.utilities.logging.Reporter;

/**
 * Interface to objects that can determines when to bid what. Implementors are
 * assumed to model all opponents using the provided {@link OpponentModel}, and
 * to evaluate the offered bids using the provided {@link AcceptanceStrategy}.
 * <p>
 * To avoid the need for the event handling as in {@link Party}, this class
 * interfaces through different classes than Party:
 * <ul>
 * <li>Incoming "Settings" events are now
 * {@link #with(Progress, PartyId, Class, Class, Reporter)}
 * <li>Incoming notifications now {@link #with(Action)}
 * <li>Progress advancing is now triggered with {@link #with(Progress)}
 * <li>Outgoing actions are replaced with {@link #getAction()} and
 * {@link #getActionTime()}.
 * </ul>
 * <p>
 * Usually part of a {@link BoaParty}.
 * 
 * <p>
 * MUST * have an empty constructor. Implementations are supposed to be
 * immutable.
 */
public interface BiddingStrategy {

	/**
	 * @return unix time (ms since 1970 ) at which the value from
	 *         {@link #getAction()} becomes valid. This allows the strategy to
	 *         delay actions after it has got the turn, or to wait for other
	 *         events to happen. Action will be executed as soon as possible
	 *         after the indicated time.
	 */
	default Long getActionTime(NegoState state) {
		return 0l;
	}

	/**
	 * 
	 * @return the suggested next Action. Valid only after
	 *         {@link #getActionTime()} > System.currentTimeMillis().
	 */
	Action getAction(NegoState state);

}
